import josx.platform.rcx.*;

public class MyRobot {
	private int normalDarkLight;
	private boolean onLine, drive, direction;
	private final int DARKLINE_DATACOUNT = 1000;
	private final int ROTATION_TIME = 1000;
	private final int ROTATION_POWER = 5;
	private final int DRIVE_POWER = 7;

	public static void main(String[] args) throws InterruptedException {
		new MyRobot();
	}

	public MyRobot() throws InterruptedException {
		// Lichtsensor initialisieren
		Sensor.S2.setTypeAndMode(SensorConstants.SENSOR_TYPE_LIGHT,
				SensorConstants.SENSOR_MODE_PCT);
		Sensor.S2.setPreviousValue(0);
		Sensor.S2.activate();

		normalDarkLight = getNormalDarkLight() + 4;
		direction = true;

		while (!Button.RUN.isPressed()) {
			onLine = isOnLine();

			// geradeaus fahren, wenn auf der Linie
			if (onLine && !drive) {
				// damit nicht nur am Linienrand,
				// warten um Drehung zu verlngern
				sleep(100);

				drive = true;
				MotorsPower(DRIVE_POWER);
				Motor.A.forward();
				Motor.C.forward();
			}

			// Linie wurde verlassen
			else if (!onLine && drive) {
				drive = false;
				setMotorDirection();
				rotateTime(ROTATION_TIME);
				if (!onLine) {
					reverseMotor();
					rotateUntilOnLine();
				}
			}
		}
		Sensor.S2.passivate();
		Motor.A.flt();
		Motor.C.flt();

		Button.RUN.waitForPressAndRelease();
	}

	// Roboter dreht sich fr eine bestimmte Zeit
	private void rotateTime(int time) {
		int t = (int) System.currentTimeMillis() + time;
		MotorsPower(ROTATION_POWER);
		while ((int) System.currentTimeMillis() < t && !isOnLine() && !Button.RUN.isPressed());
		onLine = isOnLine() || Button.RUN.isPressed();
	}

	// Roboter dreht sich, bis er wieder auf der Linie ist
	private void rotateUntilOnLine() {
		MotorsPower(ROTATION_POWER);
		while (!isOnLine() && !Button.RUN.isPressed());
		onLine = true;
	}

	// dreht die Richtung beider Motoren um
	private void reverseMotor() {
		direction = !direction;
		Motor.A.flt();
		Motor.C.flt();
		sleep(100);
		if (direction) {
			Motor.A.forward();
			Motor.C.backward();

		} else {
			Motor.C.forward();
			Motor.A.backward();
		}
	}

	// die Richtung der Motoren fr die Drehung setzen
	private void setMotorDirection() {
		if (direction) {
			Motor.A.forward();
			Motor.C.flt();
			sleep(100);
			Motor.C.backward();
		} else {
			Motor.C.forward();
			Motor.A.flt();
			sleep(100);
			Motor.A.backward();
		}
	}

	// prfen, ob Roboter auf der Linie
	private boolean isOnLine() {
		return Sensor.S2.readValue() <= normalDarkLight;
	}

	// normales dunkles Licht ermitteln
	private int getNormalDarkLight() {
		int sum = 0;
		for (int i = 0; i <= DARKLINE_DATACOUNT; i++) {
			sum += Sensor.S2.readValue();
		}
		return sum / DARKLINE_DATACOUNT;
	}

	// Geschwindigkeit der beider Motoren setzen
	private void MotorsPower(int p) {
		Motor.A.setPower(p);
		Motor.C.setPower(p);
	}

	private void sleep(int aMilliseconds) {
		try {
			Thread.sleep(aMilliseconds);
		} catch (Exception e) {
		}
	}

}
